#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Label.h>

static XrmOptionDescRec options[] = {
};

static String fallbacks[] = {
	"*mainWindow.translations: #override\\n"
	"  <Motion>: motion_proc()\\n"
	"",
	"*menubar.translations: #override\\n"
	"  <Leave>: menubar_leaveproc()\\n"
	"",
	"*menubar.internalBorderWidth: 0",
	"*menubar*background: #CCCCCC",
	"*menubar.MenuButton.showGrip: False",
	"*menubar.orientation: horizontal",
	"*menubar.menu1.label: File",
	"*menubar.menu2.label: Help",
	"*menubar.menu1*quit.label: Quit",
	"*menubar.menu2*about.label: About",
	"*about*label: Simple Menu (c)2001 T.Shiozaki",
	"*about*okbutton.label: OK",
	NULL
};

XtAppContext appcon;
Widget shell, mainwin, menubar, menu1, menu2, about;
Boolean isMenubarMapped, isEnterMenubar, isMenuPopUp;

static void
motion_proc(Widget w, XEvent *event)
{
	XMotionEvent *ev = (XMotionEvent *)event;
	Dimension width, bw, h;

	if (menubar && XtIsRealized(menubar)) {
		if (!isMenubarMapped) {
			XtVaGetValues(
				menubar,
				XtNheight, &h,
				XtNborderWidth, &bw,
				NULL);
			if (ev->y<h+bw*2) {
				isMenubarMapped = TRUE;
				XtVaGetValues(
					mainwin,
					XtNwidth, &width,
					NULL);
				XtVaSetValues(
					menubar,
					XtNx, (Position)0,
					XtNy, (Position)0,
					XtNwidth, (Dimension)(width-bw*2),
					NULL);
				XtMapWidget(menubar);
				XRaiseWindow(
					XtDisplay(menubar), XtWindow(menubar));
				isEnterMenubar = TRUE;
			}
		}
	}
}
static void
menubar_leaveproc(Widget w, XEvent *event)
{
	isEnterMenubar = FALSE;
	if (isMenubarMapped && !isMenuPopUp) {
		isMenubarMapped = FALSE;
		XtUnmapWidget(menubar);
	}
}

static XtActionsRec actions[] = {
	{ "motion_proc", (XtActionProc)motion_proc },
	{ "menubar_leaveproc", (XtActionProc)menubar_leaveproc },
};

static void
menu_popcallback(Widget w, XtPointer closure, XtPointer calldata)
{
	if (closure) {
		/* popup */
		isMenuPopUp = TRUE;
	} else {
		/* popdown */
		isMenuPopUp = FALSE;
		if (!isEnterMenubar) {
			XtUnmapWidget(menubar);
		}
	}
}

static void
menu_quit(Widget w, XtPointer closure, XtPointer calldata)
{
	exit(1);
}

static void
menu_about(Widget w, XtPointer closure, XtPointer calldata)
{
	Position mx, my;
	Dimension mw, mh, dw, dh;
	XtVaGetValues(
		shell,
		XtNx, &mx,
		XtNy, &my,
		NULL);
	XtVaGetValues(
		mainwin,
		XtNwidth, &mw,
		XtNheight, &mh,
		NULL);
	XtPopup(about, XtGrabExclusive);
	XtVaGetValues(
		about,
		XtNwidth, &dw,
		XtNheight, &dh,
		NULL);
	XtVaSetValues(
		about,
		XtNx, (Position)(mx+mw/2-dw/2),
		XtNy, (Position)(my+mh/2-dh/2),
		NULL);
}

static
about_popdown(Widget w, XtPointer closure, XtPointer calldata)
{
	XtPopdown(about);
}

int
main(int argc, char **argv)
{
	Widget smw, sme, dw, w;
	shell = XtVaOpenApplication(
		&appcon, "xawmenu", options, XtNumber(options),
		&argc, argv, fallbacks,
		applicationShellWidgetClass,
		//
		NULL);
	mainwin = XtVaCreateManagedWidget(
		"mainWindow", formWidgetClass, shell,
		//
		XtNlabel, "",
		XtNwidth, (Dimension)640,
		XtNheight, (Dimension)480,
		NULL);
	menubar = XtVaCreateWidget(
		"menubar", panedWidgetClass, mainwin,
		//
		NULL);
	menu1 = XtVaCreateManagedWidget(
		"menu1", menuButtonWidgetClass, menubar,
		//
		NULL);
	menu2 = XtVaCreateManagedWidget(
		"menu2", menuButtonWidgetClass, menubar,
		//
		NULL);
	smw = XtVaCreatePopupShell(
		"menu", simpleMenuWidgetClass, menu1,
		NULL);
	XtAddCallback(smw,
		      (String)"popupCallback",
		      (XtCallbackProc)&menu_popcallback,
		      (XtPointer)TRUE);
	XtAddCallback(smw,
		      (String)"popdownCallback",
		      (XtCallbackProc)&menu_popcallback,
		      (XtPointer)FALSE);
	sme = XtVaCreateManagedWidget(
		"dum1", smeLineObjectClass, smw,
		NULL);
	sme = XtVaCreateManagedWidget(
		"quit", smeBSBObjectClass, smw,
		NULL);
	XtAddCallback(sme,
		      (String)"callback",
		      (XtCallbackProc)&menu_quit,
		      (XtPointer)0);
	smw = XtVaCreatePopupShell(
		"menu", simpleMenuWidgetClass, menu2,
		NULL);
	sme = XtVaCreateManagedWidget(
		"dum1", smeLineObjectClass, smw,
		NULL);
	sme = XtVaCreateManagedWidget(
		"about", smeBSBObjectClass, smw,
		NULL);
	XtAddCallback(sme,
		      (String)"callback",
		      (XtCallbackProc)&menu_about,
		      (XtPointer)0);
	XtAddCallback(smw,
		      (String)"popupCallback",
		      (XtCallbackProc)&menu_popcallback,
		      (XtPointer)TRUE);
	XtAddCallback(smw,
		      (String)"popdownCallback",
		      (XtCallbackProc)&menu_popcallback,
		      (XtPointer)FALSE);
	XtVaCreateManagedWidget(
		"padding", labelWidgetClass, menubar,
		//
		XtNlabel, (String)"",
		NULL);
	about = XtVaCreatePopupShell(
		"about", transientShellWidgetClass, mainwin,
		//
		NULL);
	dw = XtVaCreateManagedWidget(
		"aboutDialog", dialogWidgetClass, about,
		//
		NULL);
	w = XtVaCreateManagedWidget(
		"okbutton", commandWidgetClass, dw,
		//
		NULL);
	XtAddCallback(w,
		      (String)"callback",
		      (XtCallbackProc)&about_popdown,
		      (XtPointer)0);
	
	XtAppAddActions(appcon, actions, XtNumber(actions));

	XtRealizeWidget(shell);
	XtRealizeWidget(menubar);
	XtAppMainLoop(appcon);
	return 0;
}
